@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -142,6 +142,7 @@ FILE * gdm_safe_fopen_ap (const char *file);
    limits */
 void gdm_get_initial_limits (void);
 void gdm_reset_limits (void);
+void gdm_reset_locale (void);
 
 const char *gdm_root_user (void);
 
@@ -179,6 +180,8 @@ const char * gdm_console_translate (const char *str);
 /* Use with C_(N_("foo")) to make gettext work it out right */
 #define C_(x) (gdm_console_translate(x))
 
+gchar * gdm_read_default (gchar *key);
+
 #endif /* GDM_MISC_H */
 
 /* EOF */
