@@ -70,6 +70,8 @@ gboolean GDM_IS_LOCAL          = FALSE;
 static gboolean ignore_buttons = FALSE;
 gboolean GdmHaltFound          = FALSE;
 gboolean GdmRebootFound        = FALSE;
+gboolean *GdmCustomCmdsFound   = NULL;
+gboolean GdmAnyCustomCmdsFound = FALSE;
 gboolean GdmSuspendFound       = FALSE;
 gboolean GdmConfiguratorFound  = FALSE;
 
@@ -83,6 +85,7 @@ extern GtkButton *gtk_ok_button;
 extern GtkButton *gtk_start_again_button;
 
 gboolean greeter_probably_login_prompt = FALSE;
+static gboolean first_prompt = TRUE;
 
 static void process_operation (guchar opcode, const gchar *args);
 
@@ -172,7 +175,10 @@ process_operation (guchar       op_code,
     static GnomeCanvasItem *disabled_cover = NULL;
     gchar *language;
     gchar *selected_user = NULL;
-
+    gint lookup_status = SESSION_LOOKUP_SUCCESS;
+    gchar *firstmsg = NULL;
+    gchar *secondmsg = NULL;
+    gint save_session = GTK_RESPONSE_NO;
 
     /* Parse opcode */
     switch (op_code) {
@@ -192,40 +198,33 @@ process_operation (guchar       op_code,
 					gdm_config_get_string (GDM_KEY_SOUND_ON_LOGIN_FILE),
 					gdm_config_get_bool (GDM_KEY_SOUND_ON_LOGIN));
 		greeter_probably_login_prompt = TRUE;
-		if (gtk_start_again_button != NULL)
-	                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
-	} else {
-		greeter_probably_login_prompt = FALSE;
-		if (gtk_start_again_button != NULL)
-                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
 	}
 	if (gtk_ok_button != NULL)
-                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), FALSE);
+
+	if (gtk_start_again_button != NULL)
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_start_again_button), !first_prompt);
+
+	first_prompt = FALSE;
 
 	greeter_ignore_buttons (FALSE);
 
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
 	g_free (tmp);
-
-	/* Reselect selected user if any */
-	selected_user = greeter_item_ulist_get_selected_user ();
-	if (greeter_probably_login_prompt == TRUE && selected_user != NULL)
-		greeter_item_ulist_select_user (selected_user);
 	break;
 
     case GDM_NOECHO:
 	tmp = ve_locale_to_utf8 (args);
 
-	if (tmp != NULL && strcmp (tmp, _("Password:")) == 0) {
-		if (gtk_start_again_button != NULL)
-                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
-		greeter_probably_login_prompt = FALSE;
-	} else {
-		if (gtk_start_again_button != NULL)
-                	gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
-	}
+	greeter_probably_login_prompt = FALSE;
+
 	if (gtk_ok_button != NULL)
-                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), FALSE);
+
+	if (gtk_start_again_button != NULL)
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_start_again_button), !first_prompt);
+
+	first_prompt = FALSE;
 
 	greeter_ignore_buttons (FALSE);
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, FALSE);
@@ -276,9 +275,60 @@ process_operation (guchar       op_code,
 
     case GDM_SESS:
 	tmp = ve_locale_to_utf8 (args);
-	session = gdm_session_lookup (tmp);
+	session = gdm_session_lookup (tmp, &lookup_status);
+	if (lookup_status != SESSION_LOOKUP_SUCCESS) {
+		switch (lookup_status) {
+		case SESSION_LOOKUP_PREFERRED_MISSING:
+			firstmsg = g_strdup_printf (_("Do you wish to make %s the default for "
+						      "future sessions?"),
+						    gdm_session_name (tmp));	    
+			secondmsg = g_strdup_printf (_("Your preferred session type %s is not "
+						       "installed on this computer."),
+						     gdm_session_name (gdm_get_default_session ()));
+			save_session = gdm_wm_query_dialog (firstmsg, secondmsg,
+							    _("Make _Default"), _("Just _Log In"), TRUE);
+			
+			g_free (firstmsg);
+			g_free (secondmsg);
+			gdm_set_save_session (save_session);
+			break;
+			
+		case SESSION_LOOKUP_DEFAULT_MISMATCH:
+			firstmsg = g_strdup_printf (_("Do you wish to make %s the default for "
+						      "future sessions?"),
+						    gdm_session_name (session));
+			secondmsg = g_strdup_printf (_("You have chosen %s for this "
+						       "session, but your default "
+						       "setting is %s."),
+						     gdm_session_name (session),
+						     gdm_session_name (tmp));
+			save_session = gdm_wm_query_dialog (firstmsg, secondmsg,
+							    _("Make _Default"), _("Just For _This Session"), TRUE);
+			
+			g_free (firstmsg);
+			g_free (secondmsg);
+			gdm_set_save_session (save_session);
+			break;
+		case SESSION_LOOKUP_USE_SWITCHDESK:
+			firstmsg = g_strdup_printf (_("You have chosen %s for this "
+						      "session"),
+						    gdm_session_name (session));
+			secondmsg = g_strdup_printf (_("If you wish to make %s "
+						       "the default for future sessions, "
+						       "run the 'switchdesk' utility "
+						       "(System->Desktop Switching Tool from "
+						       "the panel menu)."),
+						     gdm_session_name (session));			 
+			gdm_wm_message_dialog (firstmsg, secondmsg);			 
+			g_free (firstmsg);
+			g_free (secondmsg);
+			break;
+			
+		default:
+			break;
+		}		
+	}
 	g_free (tmp);
-
 	if (gdm_get_save_session () == GTK_RESPONSE_CANCEL) {
 	     printf ("%c%s\n", STX, GDM_RESPONSE_CANCEL);
 	} else {
@@ -324,9 +374,11 @@ process_operation (guchar       op_code,
     case GDM_RESETOK:
 
 	if (gtk_ok_button != NULL)
-                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_ok_button), FALSE);
 	if (gtk_start_again_button != NULL)
-                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
+                gtk_widget_set_sensitive (GTK_WIDGET (gtk_start_again_button), FALSE);
+
+	first_prompt = TRUE;
 
 	conversation_info = greeter_lookup_id ("pam-conversation");
 	
@@ -739,6 +791,8 @@ gdm_set_welcomemsg (void)
 static void
 gdm_read_config (void)
 {
+	gint i;
+
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
 
@@ -795,6 +849,28 @@ gdm_read_config (void)
 	gdm_config_get_bool   (GDM_KEY_SOUND_ON_LOGIN);
 	gdm_config_get_bool   (GDM_KEY_DEFAULT_WELCOME);
 	gdm_config_get_bool   (GDM_KEY_DEFAULT_REMOTE_WELCOME);
+	gdm_config_get_bool   (GDM_KEY_ADD_GTK_MODULES);
+
+	/* Keys for custom commands */
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+		gchar * key_string = NULL;
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+        
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+		gdm_config_get_string (key_string);
+
+		g_free (key_string);
+	}     
 
 	/* Keys not to include in reread_config */
 	gdm_config_get_string (GDM_KEY_SESSION_DESKTOP_DIR);
@@ -806,6 +882,9 @@ gdm_read_config (void)
 static gboolean
 greeter_reread_config (int sig, gpointer data)
 {
+	gint i;
+	gboolean custom_changed = FALSE;
+
 	/* Read config data in bulk */
 	gdmcomm_comm_bulk_start ();
 
@@ -853,7 +932,8 @@ greeter_reread_config (int sig, gpointer data)
 	    gdm_config_reload_bool   (GDM_KEY_GRAPHICAL_THEME_RAND) ||
 	    gdm_config_reload_bool   (GDM_KEY_SHOW_LAST_SESSION) ||
 	    gdm_config_reload_bool   (GDM_KEY_ALLOW_ROOT) ||
-	    gdm_config_reload_bool   (GDM_KEY_ALLOW_REMOTE_ROOT)) {
+	    gdm_config_reload_bool   (GDM_KEY_ALLOW_REMOTE_ROOT) ||
+	    gdm_config_reload_bool   (GDM_KEY_ADD_GTK_MODULES)) {
 
 		/* Set busy cursor */
 		gdm_common_setup_cursor (GDK_WATCH);
@@ -864,6 +944,41 @@ greeter_reread_config (int sig, gpointer data)
 		_exit (DISPLAY_RESTARTGREETER);
 	}
 
+	for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+		gchar *key_string = NULL;
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LABEL_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_LR_LABEL_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEXT_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+		
+		key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TOOLTIP_TEMPLATE, i);
+		if (gdm_config_reload_string (key_string))
+			custom_changed = TRUE;
+
+		g_free (key_string);
+	}     	
+	
+	if(custom_changed){
+		/* Set busy cursor */
+		gdm_common_setup_cursor (GDK_WATCH);
+		
+		gdm_wm_save_wm_order ();
+		gdmcomm_comm_bulk_stop ();
+		
+		_exit (DISPLAY_RESTARTGREETER);
+	}
+
 	gdm_config_reload_string (GDM_KEY_SOUND_PROGRAM);
 	gdm_config_reload_bool   (GDM_KEY_SOUND_ON_LOGIN);
 	gdm_config_reload_string (GDM_KEY_SOUND_ON_LOGIN_FILE);
@@ -1046,6 +1161,7 @@ main (int argc, char *argv[])
   const char *gdm_gtk_theme;
   guint sid;
   int r;
+  gint i;
 
   if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
     DOING_GDM_DEVELOPMENT = TRUE;
@@ -1063,9 +1179,18 @@ main (int argc, char *argv[])
   else
 	   GDM_IS_LOCAL = TRUE;
 
+  /*
+   * gdm_common_atspi_launch () needs gdk initialized.
+   * We cannot start gtk before the registry is running 
+   * because the atk-bridge will crash.
+   */
+  gdk_init (&argc, &argv);
+  if (! DOING_GDM_DEVELOPMENT) {
+       gdm_common_atspi_launch ();
+  }
+  
   gtk_init (&argc, &argv);
 
-  /* Should be a watch already, but anyway */
   gdm_common_setup_cursor (GDK_WATCH);
 
   /* Read all configuration at once, so the values get cached */
@@ -1168,13 +1293,24 @@ main (int argc, char *argv[])
   * is displayed it doesn't show as 0.  Also determine if the Halt,
   * Reboot, Suspend and Configurator commands work.
   */
-  gdm_timed_delay      = gdm_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
-  GdmHaltFound         = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_HALT));
-  GdmRebootFound       = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_REBOOT));
-  GdmSuspendFound      = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_SUSPEND));
-  GdmConfiguratorFound = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR));
+  gdm_timed_delay         = gdm_config_get_int (GDM_KEY_TIMED_LOGIN_DELAY);
+  GdmHaltFound            = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_HALT));
+  GdmRebootFound          = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_REBOOT));
+  GdmSuspendFound         = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_SUSPEND));
+  GdmConfiguratorFound    = gdm_working_command_exists (gdm_config_get_string (GDM_KEY_CONFIGURATOR));
 
+  GdmCustomCmdsFound = g_new0 (gboolean, GDM_CUSTOM_COMMAND_MAX);
+  for (i = 0; i < GDM_CUSTOM_COMMAND_MAX; i++) {
+	  gchar * key_string = NULL;
+	  /*  For each possible custom command */      
+	  key_string = g_strdup_printf(_("%s%d="), GDM_KEY_CUSTOM_CMD_TEMPLATE, i);
+	  GdmCustomCmdsFound[i] = gdm_working_command_exists (gdm_config_get_string (key_string));
+	  if (GdmCustomCmdsFound[i])
+		  GdmAnyCustomCmdsFound = TRUE;
 
+	  g_free (key_string);
+  }
+  
   if (g_getenv ("GDM_THEME") != NULL)
      gdm_graphical_theme = g_strdup (g_getenv ("GDM_THEME"));
   else if (gdm_config_get_bool (GDM_KEY_GRAPHICAL_THEME_RAND))
@@ -1463,7 +1599,7 @@ main (int argc, char *argv[])
   gdm_wm_restore_wm_order ();
 
   gdm_wm_show_info_msg_dialog (gdm_config_get_string (GDM_KEY_INFO_MSG_FILE),
-     gdm_config_get_string (GDM_KEY_INFO_MSG_FONT));
+	gdm_config_get_string (GDM_KEY_INFO_MSG_FONT));
 
   gdm_common_setup_cursor (GDK_LEFT_PTR);
   gdm_wm_center_cursor ();
